﻿using System;
using UnityEngine;
using UnityEditor;
using UnityEditorInternal;

[CustomPropertyDrawer(typeof(ReorderableGameObjectList), true)]
public class ReorderableGameObjectListDrawer : UnityEditor.PropertyDrawer
{
	private UnityEditorInternal.ReorderableList m_list;

    /// <summary>
    /// The standard height size for each property line.
    /// </summary>

    const int lineHeight = 16;

    private UnityEditorInternal.ReorderableList getList(SerializedProperty property)
	{
		if(m_list == null) {
			m_list = new ReorderableList(property.serializedObject, property, true, true, true, true);
			m_list.drawElementCallback += (Rect rect, int index, bool selected, bool focused) =>
			{
				SerializedProperty prop = m_list.serializedProperty.GetArrayElementAtIndex(index);
				EditorGUI.PropertyField(rect, prop, GUIContent.none);
			};
		}

        return m_list;
	}

	public override float GetPropertyHeight(SerializedProperty property, UnityEngine.GUIContent label)
	{
        var list = getList(property.FindPropertyRelative("List"));
		if(list != null) {
			try {
				return list.GetHeight();
			}
			catch(System.Exception e) {
				Debug.LogException(e);
				m_list = null;
				return  EditorGUIUtility.singleLineHeight;
			}
		} else {
			return  EditorGUIUtility.singleLineHeight;
		}
	}

	public override void OnGUI(UnityEngine.Rect position, SerializedProperty property, UnityEngine.GUIContent label)
	{
        var list = getList(property.FindPropertyRelative("List"));
        if(list != null)
        {
            list.drawHeaderCallback = rect =>
            {
                EditorGUI.LabelField(rect, property.name);
            };

            list.DoList(position);
        }
        
	}
}
